#version 330 core

// Interpolated values from the vertex shaders
in vec2 vTexCoords;
in vec3 vPosWorld;
in vec3 vNormalCam;
in vec3 vEyeDirCam;
in vec3 vTangent;
in vec3 vBitangent;
struct Light {
    float fLighIntensity;
    vec3 vLightColor;
    vec3 vPosition;
    vec3 vAttenuation;
    vec3 vSpotLight;
    vec3 vDirection;
    int iLightType;
};
uniform Light lights[10];
uniform int numberOfLights;
// Ouput data
out vec4 vColorOut;
// Values that stay constant for the whole mesh.
uniform sampler2D diffuseTexture;
uniform mat4 MV;
uniform mat4 V;
uniform vec3 vEyePos;
//shadow stuff
in vec4 ShadowCoord;
uniform sampler2D shadowMap;
float linstep(float low, float high, float v){
    return clamp((v-low)/(high-low), 0.0, 1.0);
}

//normal map
uniform sampler2D normalMap;
uniform bool hasNormalMap;
uniform bool hasDiffuseTex;
uniform vec3 vMaterialDiffuse;
uniform bool isSun;

vec3 calcBumpedNormal()
{
    vec3 bumpMapNormal = texture(normalMap,vTexCoords).xyz;
    bumpMapNormal = 2.0 * bumpMapNormal - vec3(1.0,1.0,1.0);
    mat3 tbn;
	// Calculate handedness and the resulting tangent, bitangent, normal matrix
	float temp = dot(cross(vNormalCam, vTangent), vBitangent);
	if(temp < 0.f) 
	{
		tbn = mat3((vTangent * -1.f), (vBitangent) , (vNormalCam));
	}
	else 
	{
		tbn = mat3((vTangent), (vBitangent), (vNormalCam));
	}
	vec3 newNormal = vec3(0.0,0.0,0.0);
    newNormal = tbn * bumpMapNormal;
    newNormal = normalize(newNormal);
	return newNormal;
}


vec4 calculateBlinnPhong(vec4 lightColor, vec4 vMaterialTexture, vec3 N, vec3 L, vec3 H )
{
    // TODO NOT HARDCODED
	// vec4 is vMaterialDiffuse
	vec4 Id = hasDiffuseTex ? (vMaterialTexture * vec4(0.992157, 0.992157, 0.992157, 1.0) *clamp( dot(N,L), 0,1 )) : (vec4(vMaterialDiffuse, 1.0) *clamp( dot(N,L), 0,1 ));
    // vec4 is vMaterialSpecular, pow exp is shininess
	vec4 Is = vec4(0.225, 0.225, 0.225, 1.0) * pow( clamp(dot(N,H), 0.0, 1.0), 33.0f );
    return (Id + Is) * lightColor;
}


void main(){
    // Eye vector (towards the camera)
	vec3 E = normalize(vEyeDirCam);
    vColorOut = vec4(0.0);
    vec4 vMaterialTexture = hasDiffuseTex ? texture(diffuseTexture, vTexCoords) : vec4(1.f);
    // should not be hardcoded, but for now sufficient
	vec4 vLightAmbient	= vec4(0.2f, 0.2f, 0.2f, 1.0f);
    vec4 vMaterialAmbient = vec4(0.25f);
    vColorOut = vMaterialAmbient * vLightAmbient;

	vec3 n = normalize(vNormalCam);

    if(hasNormalMap)
	{
		// calc normals here
        n = calcBumpedNormal();
    }

    vec3 vEye = normalize(vEyePos - vPosWorld);
    for (int i = 0; i < numberOfLights; i++) 
	{
        if(lights[i].iLightType == 0) 
		{
            // light dir/pos should be the same? thats why we abuse the position here
			vec3 l = normalize( lights[i].vPosition );
            vec3 h = normalize( vEye + l );
            // shadow should be added here, more like multiplied
			vColorOut += calculateBlinnPhong( vec4(lights[i].vLightColor, 1.0f) , vMaterialTexture, n, l, h );
        }
		else if(lights[i].iLightType == 1)
		{
            // light dir
            vec3 d = vec3( lights[i].vPosition - vPosWorld );
            // light distance
			float  dl = length(d);
            d = d/dl;
            // normalized light dir					
			vec3 h = normalize( vEye + d );
            float attenuation = 1.0f / ( lights[i].vAttenuation.x + lights[i].vAttenuation.y * dl + lights[i].vAttenuation.z * dl * dl );
            vColorOut += attenuation * calculateBlinnPhong( vec4(lights[i].vLightColor, 1.0f) , vMaterialTexture, n, d, h );
        }
		else if (lights[i].iLightType == 2) 
		{
            // implementation needs work 
			// inspiration http://ogldev.atspace.co.uk/www/tutorial21/tutorial21.html
			// light dir
            vec3 d = vec3( lights[i].vPosition - vPosWorld );
            // light distance
			float  dl = length(d);
            d = d/dl;
            // normalized light dir					
			vec3 h = normalize( vEye + d );
            vec3 sd = normalize(lights[i].vDirection);
            float rho = dot(-d, sd);
            float spot = pow( clamp( ((rho - lights[i].vSpotLight.x) / (lights[i].vSpotLight.y- lights[i].vSpotLight.x)), 0, 1) , lights[i].vSpotLight.z);
            float attenuation = spot / ( lights[i].vAttenuation.x + lights[i].vAttenuation.y * dl + lights[i].vAttenuation.z * dl * dl );
            vColorOut += attenuation * calculateBlinnPhong( vec4(lights[i].vLightColor, 1.0f) , vMaterialTexture, n, d, h );
        }

    }
	//vColorOut.rgb = n;
	// material diffuse.a
	vColorOut.a = 1.0;
}




