#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition_modelspace;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 vertexNormal_modelspace;
layout(location = 3) in vec3 tangents;

// Output data ; will be interpolated for each fragment.
out vec2 vTexCoords;
// in world space
out vec3 vPosWorld;
// both in camera space
out vec3 vNormalCam;
out vec3 vEyeDirCam;
//tangent
out vec3 vTangent;
out vec3 vBitangent;

// Values that stay constant for the whole mesh.
uniform mat4 MVP;
uniform mat4 V;
uniform mat4 M;
uniform mat4 mNormalMatrix;

//normal map
uniform sampler2D normalMap;
uniform bool hasNormalMap;


void main(){

	// Output position of the vertex, in clip space : MVP * position
	gl_Position =  MVP * vec4(vertexPosition_modelspace,1);
	
	// model matrix mulitplication brings vertex from model into world
	vPosWorld = vec3(M * vec4(vertexPosition_modelspace,1));
	
	// Vector that goes from the vertex to the camera, in camera space.
	vec3 vertexPosition_cameraspace = ( V * M * vec4(vertexPosition_modelspace,1)).xyz;
	vEyeDirCam = vec3(0,0,0) - vertexPosition_cameraspace;
	
	// Normal of the the vertex, in camera space
	vNormalCam = vec3(M * vec4(vertexNormal_modelspace,0)); 
	vTexCoords = vertexUV;
	
	//transform tangent to worldspace
	vTangent = (M * vec4(tangents, 0.0)).xyz;
	vBitangent = cross(vTangent, vNormalCam); 
}

