#version 330 core

in vec2 vTexCoords;
uniform vec2 ScaleU;
uniform sampler2D textureSource;
//vec4 color;

//out vec4 outColor;

//const vec2 gaussFilter[] =vec2[]
//(
//    vec2(-4.0, 0.0162162162),
//    vec2(-3.0, 0.0540540541),
//	vec2(-2.0, 0.1216216216),
//	vec2(-1.0,	0.1945945946),
//	vec2(0.0,	0.2270270270),
//	vec2(1.0,	0.1945945946),
//	vec2(2.0,	0.1216216216),
//	vec2(3.0,	0.0540540541),
//	vec2(4.0, 0.0162162162)
//);

//float Gaussian (float x, float deviation)
//{
//    return (1.0 / sqrt(2.0 * 3.141592 * deviation)) * exp(-((x * x) / (2.0 * deviation)));
//}

//void main()
//{
//    color = vec4(0.0);
//    for( int i = 0; i < 9; i++ )
//		{
//        int f = i % 9;
//        color += texture( textureSource,  vec2(vTexCoords.x + gaussFilter[f].x * ScaleU.x, vTexCoords.y + gaussFilter[f].x * ScaleU.y ) ) * gaussFilter[f].y;
//    }

//    outColor = color;
//}

#define MAX_BLUR_RADIUS 4096

uniform int uBlurRadius = 16;

out vec4 fResult;

/*----------------------------------------------------------------------------*/
/*	Incremental, forward-differencing Gaussian elimination based on:
	http://http.developer.nvidia.com/GPUGems3/gpugems3_ch40.html */
vec4 incrementalGauss1D(
	in sampler2D srcTex, 
	in vec2 srcTexelSize, 
	in vec2 origin,
	in int radius,
	in vec2 direction
) {

	int nSamples = clamp(radius, 1, int(MAX_BLUR_RADIUS)) / 2;
	
	if (nSamples == 0)
		return texture(srcTex, origin);
	
	 float SIGMA = float(uBlurRadius) / 8.0;
	float sig2 = SIGMA * SIGMA;
	const float TWO_PI	= 6.2831853071795;
	const float E			= 2.7182818284590;
		
//	set up incremental counter:
	vec3 gaussInc;
	gaussInc.x = 1.0 / (sqrt(TWO_PI) * SIGMA);
	gaussInc.y = exp(-0.5 / sig2);
	gaussInc.z = gaussInc.y * gaussInc.y;
	
//	accumulate results:
	vec4 result = texture(srcTex, origin) * gaussInc.x;	
	for (int i = 1; i < nSamples; ++i) {
		gaussInc.xy *= gaussInc.yz;
		
		vec2 offset = float(i) * direction * srcTexelSize;
		result += texture(srcTex, origin - offset) * gaussInc.x;
		result += texture(srcTex, origin + offset) * gaussInc.x;
	}
	
	return result;
}

/*----------------------------------------------------------------------------*/
void main() {
	vec2 texelSize = 1.0 / vec2(textureSize(textureSource, 0));
	fResult = incrementalGauss1D(textureSource, texelSize, vTexCoords, uBlurRadius, ScaleU); 
}