#version 330
in vec4 vPosition;
in vec2 vTap0;
in vec2 vTap1;
in vec2 vTap2;
in vec2 vTap3;
in vec2 vTap1Neg;
in vec2 vTap2Neg;
in vec2 vTap3Neg;

uniform sampler2D uInputTex;
uniform float horzTapOffs[7];

// Thresholds for computing sample weights
vec4 vThresh0 = vec4(0.1, 0.3, 0.5, -0.01);
vec4 vThresh1 = vec4( 0.6, 0.7, 0.8, 0.9);

out vec4 vColorOut;

void main()	
{
    // Samples
	vec4 s0, s1, s2, s3, s4, s5, s6, vWeights4;
    vec3 vWeights3, vColorSum;
    float fWeightSum;
    // Sample taps with coordinates from VS
	s0 = texture(uInputTex, vTap0);
    s1 = texture(uInputTex, vTap1);
    s2 = texture(uInputTex, vTap2);
    s3 = texture(uInputTex, vTap3);
    s4 = texture(uInputTex, vTap1Neg);
    s5 = texture(uInputTex, vTap2Neg);
    s6 = texture(uInputTex, vTap3Neg);
    // Compute weights for 4 first samples (including center tap)
	// by thresholding blurriness (in sample alpha)
	vWeights4.x = clamp(s1.a - vThresh0.x, 0.0, 1.0);
    vWeights4.y = clamp(s2.a - vThresh0.y, 0.0, 1.0);
    vWeights4.z = clamp(s3.a - vThresh0.x, 0.0, 1.0);
    vWeights4.w = clamp(s0.a - vThresh0.w, 0.0, 1.0);
    // Accumulate weighted samples
	vColorSum = s0.xyz * vWeights4.x + s1.xyz * vWeights4.y + s2.xyz * vWeights4.z + s3.xyz * vWeights4.w;
    // Sum weights using DOT
	fWeightSum = dot(vWeights4, vec4(1));
    // Compute weights for 3 remaining samples
	vWeights3.x = clamp(s4.a - vThresh0.x, 0.0, 1.0);
    vWeights3.y = clamp(s5.a - vThresh0.y, 0.0, 1.0);
    vWeights3.z = clamp(s6.a - vThresh0.z, 0.0, 1.0);
    // Accumulate weighted samples
	vColorSum += s4.xyz * vWeights3.x + s4.xyz * vWeights3.y + s6.xyz * vWeights3.z;
    // Sum weights using DOT
	fWeightSum += dot(vWeights3, vec3(1));
    // Compute tex coords for other taps
	vec2 vTap4 = vTap0 + vec2(horzTapOffs[4], 0.0f);
    vec2 vTap5 = vTap0 + vec2(horzTapOffs[5], 0.0f);
    vec2 vTap6 = vTap0 + vec2(horzTapOffs[6], 0.0f);
    vec2 vTap4Neg = vTap0 - vec2(horzTapOffs[4], 0.0f);
    vec2 vTap5Neg = vTap0 - vec2(horzTapOffs[5], 0.0f);
    vec2 vTap6Neg = vTap0 - vec2(horzTapOffs[6], 0.0f);
    // Sample the taps
	s0 = texture(uInputTex, vTap4);
    s1 = texture(uInputTex, vTap5);
    s2 = texture(uInputTex, vTap6);
    s3 = texture(uInputTex, vTap4Neg);
    s4 = texture(uInputTex, vTap5Neg);
    s5 = texture(uInputTex, vTap6Neg);
    // Compute weights for 3 samples
	vWeights3.x = clamp(s0.a - vThresh1.x, 0.0, 1.0);
    vWeights3.y = clamp(s1.a - vThresh1.y, 0.0, 1.0);
    vWeights3.z = clamp(s2.a - vThresh1.z, 0.0, 1.0);
    // Accumulate weighted samples
	vColorSum += s0.xyz * vWeights3.x + s1.xyz * vWeights3.y + s2.xyz * vWeights3.z;
    // Sum weights using DOT
	fWeightSum += dot(vWeights3, vec3(1));
    // Compute weights for 3 samples
	vWeights3.x = clamp(s3.a - vThresh1.x, 0.0, 1.0);
    vWeights3.y = clamp(s4.a - vThresh1.y, 0.0, 1.0);
    vWeights3.z = clamp(s5.a - vThresh1.z, 0.0, 1.0);
    // Accumulate weighted samples
	vColorSum += s3.xyz * vWeights3.x + s4.xyz * vWeights3.y + s5.xyz * vWeights3.z;
    // Sum weights using DOT
	fWeightSum += dot(vWeights3, vec3(1));
    // Divide weighted sum of samples by sum of all weights
	vColorSum /= fWeightSum;

    // Color and weights sum output scaled (by 1/256)
	// to fit values in 16 bit 0 to 1 range
	vColorOut =vec4(vColorSum, fWeightSum);// * 0.00390625; // texture(uInputTex, vTap0);// 
}

