#version 330
in vec4 vPosition;
in vec2 vTap0;
in vec2 vTap1;
in vec2 vTap2;
in vec2 vTap3;
in vec2 vTap1Neg;
in vec2 vTap2Neg;
in vec2 vTap3Neg;

uniform sampler2D uInputTex;
uniform float vertTapOffs[7];

// Thresholds for computing sample weights
vec4 vWeights0 = vec4(0.080, 0.075, 0.070, 0.100);
vec4 vWeights1 = vec4(0.065, 0.060, 0.055, 0.050);

out vec4 vColorOut;

void main()	
{
	vec4 s0, s1, s2, s3, s4, s5, s6;
	// Acumulated color and weights
	vec4 vColorWeightSum;
	// Sample taps with coordinates from VS
	s0 = texture(uInputTex, vTap0);
	s1 = texture(uInputTex, vTap1);
	s2 = texture(uInputTex, vTap2);
	s3 = texture(uInputTex, vTap3);
	s4 = texture(uInputTex, vTap1Neg);
	s5 = texture(uInputTex, vTap2Neg);
	s6 = texture(uInputTex, vTap3Neg);
	// Modulate sampled color values by the weights stored
	// in the alpha channel of each sample
	s0.rgb = s0.rgb * s0.a;
	s1.rgb = s1.rgb * s1.a;
	s2.rgb = s2.rgb * s2.a;
	s3.rgb = s3.rgb * s3.a;
	s4.rgb = s4.rgb * s4.a;
	s5.rgb = s5.rgb * s5.a;
	s6.rgb = s6.rgb * s6.a;
	// Aggregate all samples weighting them with pre-defined
	// kernel weights, weight sum in alpha
	vColorWeightSum = s0 * vWeights0.w +
	(s1 + s4) * vWeights0.x +
	(s2 + s5) * vWeights0.y +
	(s3 + s6) * vWeights0.z;
	// Compute tex coords for other taps
	vec2 vTap4 = vTap0 + vec2(0.0, vertTapOffs[4]);
	vec2 vTap5 = vTap0 + vec2(0.0, vertTapOffs[5]);
	vec2 vTap6 = vTap0 + vec2(0.0, vertTapOffs[6]);
	vec2 vTap4Neg = vTap0 - vec2(0.0, vertTapOffs[4]);
	vec2 vTap5Neg = vTap0 - vec2(0.0, vertTapOffs[5]);
	vec2 vTap6Neg = vTap0 - vec2(0.0, vertTapOffs[6]);
	// Sample the taps
	s0 = texture(uInputTex, vTap4);
	s1 = texture(uInputTex, vTap5);
	s2 = texture(uInputTex, vTap6);
	s3 = texture(uInputTex, vTap4Neg);
	s4 = texture(uInputTex, vTap5Neg);
	s5 = texture(uInputTex, vTap6Neg);
	// Modulate sampled color values by the weights stored
	// in the alpha channel of each sample
	s0.rgb = s0.rgb * s0.a;
	s1.rgb = s1.rgb * s1.a;
	s2.rgb = s2.rgb * s2.a;
	s3.rgb = s3.rgb * s3.a;
	s4.rgb = s4.rgb * s4.a;
	s5.rgb = s5.rgb * s5.a;
	// Aggregate all samples weighting them with pre-defined
	// kernel weights, weight sum in alpha
	vColorWeightSum += (s1 + s3) * vWeights1.x +
	(s1 + s4) * vWeights1.y +
	(s2 + s5) * vWeights1.z;
	// Average combined sample for all samples in the kernel
	vColorWeightSum.rgb /= vColorWeightSum.a;
	// Account for scale factor applied in previous pass
	// (blur along the X axis) to output values
	// in 16 bit 0 to 1 range
	vColorOut = vColorWeightSum;//* 256.0;
}

